/*
    Drools5 Integration Helper
    Copyright (C) 2009  Mathieu Boretti mathieu.boretti@gmail.com

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */
package org.boretti.drools.integration.drools5.annotations;

import static java.lang.annotation.ElementType.PARAMETER;
import static java.lang.annotation.RetentionPolicy.RUNTIME;

import java.lang.annotation.Documented;
import java.lang.annotation.Retention;
import java.lang.annotation.Target;

import org.boretti.drools.integration.drools5.DroolsParameterHandling;

/**
 * This annotation defines how parameter must be handled.
 * 
 * Usage of this annotation is optional. If omitted, NONE is assumed.
 * @author mbo
 *
 */
@Documented
@Retention(RUNTIME)
@Target(PARAMETER)
public @interface DroolsParameter {
	/**
	 * This defines how to handle the parameter.
	 * <ul>
	 * <li>NONE=No special treatment</li>
	 * <li>IGNORED=Don't pas the parameter to drools</li>
	 * <li>TOFLAT1=If parameter is list or collection, pass each item in drools and not the parameter</li>
	 * <li>TOFLATALL=If parameter is list or collection, pass each item in drools and not the parameter and do it recursively</li>
	 * </ul>
	 * @return the parameter handling.
	 */
	DroolsParameterHandling handling() default DroolsParameterHandling.NONE;
}
