/*
    Drools5 Integration Helper
    Copyright (C) 2009  Mathieu Boretti mathieu.boretti@gmail.com

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */
package org.boretti.drools.integration.drools5.annotations;

import static java.lang.annotation.ElementType.METHOD;
import static java.lang.annotation.RetentionPolicy.RUNTIME;

import java.lang.annotation.Documented;
import java.lang.annotation.Retention;
import java.lang.annotation.Target;

import org.boretti.drools.integration.drools5.DroolsServiceType;

/**
 * This annotation defines that a method have pre-condition.
 * 
 * @author mbo
 * @since 1.1.0
 */
@Documented
@Retention(RUNTIME)
@Target(METHOD)
public @interface DroolsPreCondition {
	/**
	 * This is the type of the service source.
	 * <ul>
	 * <li>COMPILED=Use a compiled drools rule</li>
	 * <li>SOURCE=Use a source drools file and compile it</li>
	 * <li>XML=This is a XML source drools file and compile it</li>
	 * </ul>
	 * 
	 * @return the type of the drools resource.
	 */
	DroolsServiceType type() default DroolsServiceType.COMPILED;

	/**
	 * This is the resource name. Resource name can also be of the for
	 * http://XXX or file://XXX
	 * 
	 * @return this resource name (location)
	 */
	String resourceName();

	/**
	 * This is the error to be found or thrown.
	 * 
	 * @return the error to be look for.
	 */
	Class<? extends Error> error();
}
