/*
    Drools5 Integration Helper
    Copyright (C) 2009  Mathieu Boretti mathieu.boretti@gmail.com

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */
package org.boretti.drools.integration.drools5.annotations;

import static java.lang.annotation.ElementType.METHOD;
import static java.lang.annotation.RetentionPolicy.RUNTIME;

import java.lang.annotation.Documented;
import java.lang.annotation.Retention;
import java.lang.annotation.Target;


/**
 * This annotation defines a method that is used as a Drools Variable.
 * 
 * This must be use on a method of an interface annotated with @DroolsService.
 * 
 * Only setter method setXXX with one parameter and return type void can be annotated with this
 * Annotation.
 * 
 * This type of method are used to inject fact (when global = false) or global (global = true) 
 * into drools. The injection of the field is only done at start of a session (stateless or 
 * statefull).
 * 
 * @author mbo
 *
 */
@Documented
@Retention(RUNTIME)
@Target(METHOD)
public @interface DroolsVariable {
	/**
	 * This is the fact this variable is global. 
	 * 
	 * By default it is false.
	 * 
	 * When global is true, the variable is asserted as global. When not global, the
	 * variable is asserted as "normal" variable.
	 * @return The fact this variable is global or not (global as defined by Drools)
	 */
	boolean global() default false;
}
