/*
    Drools5 Integration Helper
    Copyright (C) 2009  Mathieu Boretti mathieu.boretti@gmail.com

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */
package org.boretti.drools.integration.drools5.implementation;

import java.lang.annotation.Annotation;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.apache.log4j.Logger;
import org.boretti.drools.integration.drools5.DroolsServiceType;
import org.boretti.drools.integration.drools5.annotations.DroolsService;
import org.drools.RuleBase;
import org.drools.StatelessSession;

/**
 * @author mbo
 *
 */
class DroolsImplementationHelper {
	
	private static final Logger logger = Logger.getLogger(DroolsImplementationHelper.class);

	private DroolsImplementationHelper() {}
	
	static void validNullability(Object o,String msg) {
		if (o==null) {
			logger.error(msg);
			throw new IllegalArgumentException(msg);
		}
	}
	
	/**
	 * @param droolsProviderImplementation
	 * @param clazz
	 * @param type
	 * @param resourceName
	 * @param error
	 * @param reference
	 * @param args
	 * @since 1.2.0
	 */
	@SuppressWarnings("unchecked")
	static <T> void runCondition(DroolsProviderImplementation droolsProviderImplementation,
			Class<T> clazz,
			DroolsServiceType type,
			String resourceName,
			Class<? extends Error> error,
			Object reference,
			Object args[]) {
		validNullability(error,"Error can't be null");
		validNullability(reference,"Reference can't be null");
		validNullability(args,"Args can't be null");
		if (logger.isDebugEnabled()) logger.debug("Doing PreCondition check for "+clazz+":"+type+":"+resourceName+":"+error);
		RuleBase rb = droolsProviderImplementation.getRuleBase(clazz,type,resourceName);
		Logger l = Logger.getLogger(clazz);
		StatelessSession ss = rb.newStatelessSession();
		new DroolsWorkingMemoryLogger(ss);
		ss.setGlobal(DroolsProviderImplementation.FIELD_NAME_LOGGER, l);
		List<Object> al = new ArrayList<Object>();
		al.add(reference);
		for(Object o:args) al.add(o);
		Iterator<Object> i = ss.executeWithResults(al).iterateObjects();
		while(i.hasNext()) {
			Object o = i.next();
			if (error.isAssignableFrom(o.getClass())) {
				Error e = error.cast(o);
				logger.error("Error thrown from pre-condition : "+e,e);
				throw e;
			}
		}
	}
	
	static <T> DroolsService getAnnotation(Class<T> clazz) {
		Annotation as[] = clazz.getAnnotations();
		for(Annotation a:as) {
			if (a.annotationType().equals(DroolsService.class)) return (DroolsService)a;
		}
		return null;
	}
	
	static <T> DroolsInterfaceImplementation<T> getDroolsImplementation(DroolsServiceType dst,String resource,Class<T> clazz) {
		if (DroolsServiceType.COMPILED.equals(dst)) {
			if (resource==null || resource.trim().equalsIgnoreCase("")) resource = clazz.getName()+".cdrl";
			return new DroolsCompiledInterfaceImplementation<T>(clazz, resource);
		} else if (DroolsServiceType.SOURCE.equals(dst)){
			if (resource==null || resource.trim().equalsIgnoreCase("")) resource = clazz.getName()+".drl";
			return new DroolsSourceInterfaceImplementation<T>(clazz,resource);
		} else {
			if (resource==null || resource.trim().equalsIgnoreCase("")) resource = clazz.getName()+".xml";
			return new DroolsXMLInterfaceImplementation<T>(clazz,resource);
		}
	}
}
